import threading
import wx
import wx.lib.inspection
from pip._vendor.contextlib2 import contextmanager
from unittest import TestCase
from unittest.mock import MagicMock

from rewx import create_element, wsx, render, patch
import rewx.components as c
from tests import images

class TestStaticBitmap(TestCase):

    image_size = (32, 32)

    def test_gauge(self):
        cases  = [
            # {'uri': None},
            {},
            {'uri': images.red_uri},
            {'uri': images.red_uri, 'size': (64, 64)},
            {'uri': images.red_uri, 'size': (16, 16)}
        ]
        # expected_state = {
        #     'uri': images.kirbs_uri
        # }
        app = wx.App()
        for props in cases:
            with self.subTest(props):
                component = wsx(
                    [c.Frame, {'show': True, 'size': (100,100)},
                     [c.Block, {},
                      [c.StaticBitmap, {**props, 'proportion': 0}]]])
            frame = render(component, None)
            bitmap = frame.Children[0].Children[0]

            if 'uri' in props:
                self.assertIsNotNone(bitmap.GetBitmap().RefData)
            else:
                # when no URI is defined, there should similarly
                # be no bitmap data present
                self.assertEqual(bitmap.GetBitmap().RefData, None)

    def testUriUpdatesChangeImage(self):
        """
        Testing that swapping URI leads to the
        new images being loaded.
        """
        app = wx.App()

        with self.subTest("initializing with no URI"):
            component = wsx(
                [c.Frame, {'show': True, 'size': (100, 100)},
                 [c.StaticBitmap, {}]])
            frame = render(component, None)
            bitmap = frame.Children[0]
            # no images loaded
            self.assertEqual(bitmap.GetBitmap().RefData, None)

        with self.subTest('Adding a URI loads the image'):
            patch(bitmap, create_element(c.StaticBitmap, {'uri': images.red_uri}))
            original_rgb = self.get_rgb(bitmap)
            self.assertEqual(original_rgb, images.red_rgb)

        with self.subTest("updating props with new URI"):
            patch(bitmap, create_element(c.StaticBitmap, {'uri': images.pink_uri}))
            updated_rgb = self.get_rgb(bitmap)
            # colors should now be different because the image was updated
            self.assertNotEqual(original_rgb, updated_rgb)
            # and we should have our pink color:
            self.assertEqual(updated_rgb, images.pink_rgb)

        with self.subTest("updating props with same URI keeps existing image"):
            # attaching a mock so we can assert the same URI doesn't cause
            # a reloading of the image
            mock = MagicMock()
            bitmap.SetBitmap = mock

            patch(bitmap, create_element(c.StaticBitmap, {'uri': images.pink_uri}))
            mock.assert_not_called()


    def get_rgb(self, bitmap: wx.StaticBitmap):
        """Grab the rgb value of the first color in the image"""
        bytes = wx.ImageFromBitmap(bitmap.GetBitmap()).GetData()
        return tuple([x for x in bytes][:3])



import wx
from unittest import TestCase

from rewx.components import NotebookItem
from rewx import mount, create_element
from rewx import components


def exclude(m, keys):
    return {k: v for k,v in base_props.items()
           if k not in keys}

base_props = {
    'label': 'some label',
    'value': 'some value',
    'background_color': '#ff00ff',
    'foreground_color': '#ff00ff',
    'font': wx.Font(),
    'helptext': 'some help text',
    'name': type.__name__,
    'min_size': (100, 100),
    'max_size': (200, 200),
    'tooltip': 'Tool tip here!',
    'enabled': True,
    'show': True,
}

valid_props = {
    wx.ListCtrl: {
        **exclude(base_props, {'value', 'background_color'}),
        'column_def': [],
        'data': []
    },
    wx.Panel: {
        **exclude(base_props, {'value'}),
    },
    wx.RadioBox: {
        **exclude(base_props, {'value'}),
        'on_change': lambda *args, **kwargs: None,
        'selected': 1,
        'choices': ['one', 'two'],
        'enabled_items': [0, 1]
    },
    wx.RadioButton: {
        **exclude(base_props, {'value'})
    },
    wx.ListBox: {
        **exclude(base_props, {'value'}),
        'choices': ['a', 'b'],
        'selected': [0, 1]
    },
    wx.Gauge: {
        **base_props,
        'value': 1
    },
    wx.Slider: {
        **base_props,
        'value': 10,
        'min': 0,
        'max': 1000
    },
    wx.SpinCtrl: {
        **base_props,
        'value': 10,
        'min': 0,
        'max': 1000
    },
    wx.SpinCtrlDouble: {
        **base_props,
        'value': 10,
        'min': 0,
        'max': 1000,
        'increment': 0.01,
        'digits': 5
    },
    wx.Notebook: {
        **exclude(base_props, {'value'})
    },
    wx.StaticBox: {
        **exclude(base_props, {'value'})
    },
    wx.StaticLine: {
        **exclude(base_props, {'value'})
    },
    wx.StaticText: {
        **exclude(base_props, {'value'})
    },
    components.Grid: {
        **exclude(base_props, {'value'})
    },
    components.Block: {
        **exclude(base_props, {'value'})
    }
}



class TestWidgets(TestCase):

    def get_defined_components(self):
        required_props = (
            NotebookItem,
        )
        return [
            obj for name, obj in components.__dict__.items()
            if name[0].isupper() and obj not in required_props]

    def empty_element(self, type):
        return create_element(type, {})

    def populated_element(self, type):
        return create_element(type, valid_props.get(type, {**base_props}))

    def test_empty_mounting(self):
        """
        Testing that all supported WX Primitives can
        be mounted without additional supporting props (excluding
        the RadioBox case).
        """
        app = wx.App()
        parent = wx.Frame(None)
        for element in map(self.empty_element, self.get_defined_components()):
            with self.subTest(element):
                # RadioBox doesn't allow constructions without `choices`
                # being present
                if element['type'] == wx.RadioBox:
                    element['props']['choices'] = ['a', 'b', 'c']
                instance = mount(element, parent)
                self.assertIsNotNone(instance)
                self.assertIsInstance(instance, element['type'])


    def test_basic_props(self):
        app = wx.App()
        parent = wx.Frame(None)
        for element in map(self.populated_element, self.get_defined_components()):
            with self.subTest(element):
                # RadioBox doesn't allow constructions without `choices`
                # being present
                if element['type'] == wx.RadioBox:
                    element = self.populated_element(wx.RadioBox)
                    element['props']['choices'] = ['a', 'b', 'c']
                elif element['type'] == wx.CheckBox:
                    element = self.populated_element(wx.CheckBox)
                    element['props']['value'] = True
                instance = mount(element, parent)
                self.assertIsNotNone(instance)
                self.assertIsInstance(instance, element['type'])


import wx
from unittest import TestCase

from rewx import create_element, wsx, Component, render
import rewx.components as c

class Parent(Component):
    def __init__(self, props):
        super().__init__(props)
        self.state = {
            'label': 'initial'
        }

    def update_label(self, new_value):
        self.set_state({'label': str(new_value)})

    def render(self):
        return wsx([c.Frame, {'show': True},
                    [Nested, {'label': self.state}],
                    [c.Button, {'label': 'click', 'on_click': self.update_label}]])

class Nested(Component):
    def __init__(self, props):
        super().__init__(props)

    def render(self):
        return create_element(c.StaticText, self.props['label'])

if __name__ == '__main__':
    app = wx.App()
    frame = render(create_element(Parent, {}), None)
    app.MainLoop()


#
# from random import random, randint
#
# import math
#
# #
# # for i in range(1, 15):
# #     res = math.pow((1 - .3), i)
# #     nothit = 1 - res
# #     print(f"Odds to hit after {i} shots: {nothit * 100}")
#
#
#
#
# print(1 - math.pow(5.0/6, 3))
#
# '''
# 1 - (5/6)^n = 0.42129629629629617
# '''
#
import pandas as pd



xs = '''INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1991	16	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1992	10	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1993	12	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1994	1	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1996	6	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1998	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	1999	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2001	4	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2002	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2003	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2004	4	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2005	6	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2006	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2007	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2008	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2009	12	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2010	1	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2011	7	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2012	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2013	5	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2014	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2015	1	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2016	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2017	2	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2019	1	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	2020	3	0.00%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	Unknown Date	175	0.02%
INFLUENZA VIRUS VACCINE, NO BRAND NAME (FLUX(SEASONAL))	No Death	20,020	1.72%'''



xs.split('\n')


import wx
from rewx import Component, wsx, render, create_element
from rewx.components import Block, Button, Gauge


class FooGauge(Component):
    def __init__(self, props):
        super().__init__(props)
        self.props = props
        self.state = {"counter": 1000}

    def update_count(self, event):
        self.set_state({"counter": self.state.get("counter", 0) + 100})

    def render(self):
        return wsx(
            [Block, {},
             [Gauge, {"value": self.state["counter"],
                      "range": 3000,
                      "name": "Counter",
                      "flag": wx.CENTER | wx.ALL,
                      "size": (5000, 200),  # How to set size of this Gauge?
                      "border": 30,
                      "pulse": False}],
             [Button, {"label": "Update",
                       "on_click": self.update_count,
                       "flag": wx.CENTER | wx.ALL}],
            ]
        )


if __name__ == "__main__":
    app = wx.App()

    import wx.lib.inspection
    wx.lib.inspection.InspectionTool().Show()

    frame = wx.Frame(None, title="Gauge With Update")
    clock = render(create_element(FooGauge, {}), frame)

    frame.Show()
    app.MainLoop()


import wx
from random import randint, random
from typing import Callable
from typing_extensions import TypedDict

from rewx import Component, wsx, create_element, render
import rewx.components as c


class Parent(Component):
    """
    The main 'container' component. It's the root for all
    state management. It passes behaviors down to its children, which
    in turn cause the state here to be modified when fired.
    """

    def __init__(self, props):
        super().__init__(props)
        self.state = {"some_value": randint(1, 50)}

    def update_value(self, event):
        self.set_state({"some_value": self.state.get("some_value") + 1})

    def render(self):
        return wsx(
            [
                c.Frame,
                {"show": True, "size": (200, 100)},
                [
                    c.Block,
                    {},
                    [LabeledGauge,
                        {
                            "value": self.state["some_value"],
                            "range": 100,
                            "min_size": (100, 20),
                            "size": (100, 20),
                        },
                    ],
                    [c.StaticText, {"label": str(self.state["some_value"])}],
                    # checkout the arbitrary prop we've defined here
                    [Child, {"click_handler": self.update_value, "label": "Click me!"}],
                ],
            ]
        )


class ChildProps(TypedDict):
    """Optionally typing the props of your component"""

    label: str
    click_handler: Callable[[wx.CommandEvent], None]


class Child(Component):
    """
    An arbitrary component. Note that this is still pure -- it holds
    no state of its own and all behaviors, like the click handler, are
    passed in via props.
    """

    def __init__(self, props: ChildProps):
        super().__init__(props)

    def render(self):
        return wsx(
            [
                c.Button,
                {"on_click": self.props["click_handler"], "label": self.props["label"]},
            ]
        )


class LabeledGauge(Component):
    def __init__(self, props):
        super().__init__(props)

    def render(self):
        return wsx(
            [
                c.Block,
                {
                    "orient": wx.HORIZONTAL,
                },
                [c.StaticText, {"label": str(self.props.get("value", "Download"))}],
                [
                    c.Gauge,
                    {
                        "value": self.props.get("value", 0),
                        "range": 100,
                        "min_size": (100, 20),
                    },
                ],
            ],
        )


if __name__ == "__main__":
    app = wx.App()
    frame = render(create_element(Parent, {}), None)
    app.MainLoop()


import wx

from rewx import create_element, wsx, render, Component, Ref
import rewx.components as c


class MyComponent(Component):
    def __init__(self, props):
        super().__init__(props)
        self.state = {'value': ''}
        self.input_ref = Ref()

    def component_did_mount(self):
        wx_ctrl = self.input_ref.instance
        wx_ctrl.Bind(wx.EVT_TEXT_ENTER, self.on_enter)

    def on_input(self, event):
        self.set_state({'value': event.EventObject.Value})

    def on_enter(self, event):
        wx.MessageDialog(None, "Enter pressed!").ShowModal()

    def render(self):
        return wsx(
            [c.Frame, {'title': 'My Cool Application', 'show': True},
             [c.Block, {'orient': wx.HORIZONTAL},
              [c.TextCtrl, {
                  'style': wx.TE_PROCESS_ENTER,
                  'value': self.state['value'],
                  'on_change': self.on_input,
                  'ref': self.input_ref,
              }]]]
        )



if __name__ == '__main__':
    app = wx.App()
    frame = render(create_element(MyComponent, {}), None)
    app.MainLoop()


import wx
from unittest import TestCase

from rewx import create_element, wsx, Component, render
import rewx.components as c

class TestNestedComponents(TestCase):

    def test_update_component(self):
        app = wx.App()
        frame = render(create_element(Parent, {}), None)
        # app.MainLoop()
        # sanity checking that state/props align for initial render
        initial_parent_state = frame._instance.state['label']
        initial_child_props = frame.Children[0]._instance.props['label']
        self.assertEqual(initial_parent_state, initial_child_props)
        # bug #0002 was that child component weren't receiving new props
        # up update. So now we fire one of the rewx handlers:
        frame._instance.update_label('foobar')
        next_parent_state = frame._instance.state['label']
        next_child_props = frame.Children[0]._instance.props['label']
        # and expect that both are still in sync
        self.assertEqual(next_parent_state, 'foobar')
        self.assertEqual(next_parent_state, next_child_props)
        app.Destroy()



class Parent(Component):
    """Test Component"""
    def __init__(self, props):
        super().__init__(props)
        self.state = {
            'label': 'initial'
        }

    def update_label(self, new_value):
        self.set_state({'label': new_value})

    def render(self):
        return wsx([c.Frame, {'show': True},
                    [Nested, {'label': self.state['label']}]])

class Nested(Component):
    """Test Component"""
    def __init__(self, props):
        super().__init__(props)

    def render(self):
        return create_element(c.StaticText, {'label': self.props['label']})


import threading
import wx
import wx.lib.inspection
from pip._vendor.contextlib2 import contextmanager
from unittest import TestCase

from rewx import create_element, wsx, render
import rewx.components as c


class TestGaugeElement(TestCase):

    def test_gauge(self):
        expected_state = {
            'value': 123,
            'range': 3000,
            'name': 'Counter',
            'size': (500, 30),
        }
        app = wx.App()
        component = wsx(
            [c.Frame, {'show': True, 'size': (999,999)},
             [c.Block, {},
              [c.Gauge, {**expected_state, 'proportion': 0}]]])
        frame = render(component, None)
        gauge = frame.Children[0].Children[0]

        self.assertEqual(gauge.Size, expected_state['size'])
        self.assertEqual(gauge.Range, expected_state['range'])
        self.assertEqual(gauge.Name, expected_state['name'])
        self.assertEqual(gauge.Value, expected_state['value'])






import os

path = os.path.dirname(__file__)


red_uri = os.path.join(path, 'red.png')
red_rgb = (235, 0, 0)

green_uri = os.path.join(path, 'green.png')
green_rgb = (120, 73, 21)

pink_uri = os.path.join(path, 'pink.png')
pink_rgb = (255, 101, 222)


from typing import Tuple

from PIL import Image  # type: ignore
import wx  # type: ignore


def load(img_path) -> Image:
    return Image.open(img_path)


def resize_image(im: Image, size: Tuple[int, int]):
    im.thumbnail(size)
    return im


def to_bitmap(im: Image):
    try:
        rgba = im.convert('RGBA').tobytes()
    except AttributeError:
        rgba = im.convert('RGBA').tostring()

    return wx.Bitmap.FromBufferRGBA(im.size[0], im.size[1], rgba)



import wx
import wx.lib.inspection
from unittest import TestCase
from unittest.mock import MagicMock

from rewx import create_element, wsx, render
import rewx.components as c


class TestTextCtrlElement(TestCase):

    def test_textctrl(self):
        mocked_change_handler = MagicMock()
        mocked_enter_handler = MagicMock()
        expected_state = {
            'value': '123',
            'name': 'My Cool Control',
            'style': wx.TE_PROCESS_ENTER,
            'on_change': mocked_change_handler,
            'size': (100, 20)
        }

        app = wx.App()
        component = wsx(
            [c.Frame, {'show': True, 'size': (999, 999)},
             # note: the inner block is required so that the widget's
             # size prop can be honored (without it, it'd get resized to
             # the parent frame's size after `Layout` gets called.
             [c.Block, {},
              [c.TextCtrl, {**expected_state, 'proportion': 0}]]])
        frame = render(component, None)
        instance = frame.Children[0].Children[0]


        try:
            instance.Bind(wx.EVT_TEXT_ENTER, mocked_enter_handler)
        except Exception:
            self.fail("""
                Unexpected failure. 
                Either the test definition has regressed, and it is now missing 
                `wx.TE_PROCESS_ENTER` as a style argument, or the widget dispatcher 
                has regressed and is no longer applying `style` props correctly""")

        # our props are honored
        self.assertEqual(instance.Name, expected_state['name'])
        self.assertEqual(instance.Value, expected_state['value'])
        self.assertEqual(instance.Size, expected_state['size'])

        # our enter has picked up and installed
        self.assertFalse(mocked_enter_handler.called)
        event = wx.KeyEvent(wx.wxEVT_TEXT_ENTER)
        instance.ProcessEvent(event)
        self.assertTrue(mocked_enter_handler.called)

        # our change handler prop is honored
        self.assertFalse(mocked_change_handler.called)
        event = wx.CommandEvent(wx.wxEVT_TEXT, wx.Window.NewControlId())
        event.SetString('a')
        instance.ProcessEvent(event)
        self.assertTrue(mocked_change_handler.called)

